#include "opencv2/opencv.hpp"

using namespace cv;

#define PI 3.1415926

//detectando linhas
int main(int, char**)
{
    Mat original = imread("road.jpg", 0);
    Mat gaussiana, edges, binaria;
    GaussianBlur(original, gaussiana, Size(7,7), 1.5, 1.5);
    
    /* 0 - threshold inferior
     * 50 - threshold superior
     * 3 - tamanho do operador (mascara)
     */
    Canny(gaussiana, edges, 0, 50, 3);

    threshold(edges,binaria,
                    128,   //corte
                    255,   
                    cv::THRESH_BINARY);
    
    //linhas geradas pelo Hough
    
    std::vector<Vec4i> linesP;
    HoughLinesP(binaria,linesP,
                    1, //delta rho
                    PI/180, //delta theta
                    136, //mínimo de votos necessários
                    100, //tamanho mínimo
                    30 //gap máximo
            );
    
    Mat linhasHP;
    for (int n=0; n<linesP.size(); n++) {
        
        linhasHP = Mat::zeros(original.size(), CV_8U);
        
        line(linhasHP,
             Point(linesP[n][0],linesP[n][1]),
             Point(linesP[n][2],linesP[n][3]),
             Scalar(255), 5);

        //obtem as bordas
        bitwise_and(edges,linhasHP,linhasHP);

        //guarda num vetor todos os pontos que forem maior que zero
        std::vector<cv::Point> points;
        for( int y = 10; y < linhasHP.rows-10; y++ ) {
           uchar* rowPtr = linhasHP.ptr<uchar>(y);
           for( int x = 10; x < linhasHP.cols-10; x++ ) {
                if (rowPtr[x] > 0) {
                   points.push_back(cv::Point(x,y));
                }
            }       
        }

        Vec4f line;
        
        //fitting dos pontos a uma reta
        fitLine(cv::Mat(points),line,
                      CV_DIST_L2, // distance type
                      0,          // not used with L2 distance
                      0.01,0.01); // accuracy

        cv::line(original,Point(line[2],line[3]),Point(line[2]-150*line[0],line[3]-150*line[1]), Scalar(0),3);
    
        //imshow("edges", edges); waitKey(0);
        imshow("Hough P", linhasHP);   
        imshow("original", original); waitKey(0);
    
    }
    imshow("original", original); waitKey(0);
       
       
    return 0;
}


