#include "opencv2/opencv.hpp"

using namespace cv;

#define PI 3.1415926

void drawLines(std::vector<Vec2f> lines, Mat result);
void drawLinesP(std::vector<Vec4i> lines, Mat image);


//detectando linhas
int main(int, char**)
{
    Mat original = imread("road.jpg", 0);
    Mat gaussiana, edges, binaria;
    GaussianBlur(original, gaussiana, Size(7,7), 1.5, 1.5);
    
    /* 0 - threshold inferior
     * 50 - threshold superior
     * 3 - tamanho do operador (mascara)
     */
    Canny(gaussiana, edges, 0, 50, 3);

    threshold(edges,binaria,
                    128,   //corte
                    255,   
                    cv::THRESH_BINARY);
    
    //linhas geradas pelo Hough
    
    
    Mat linhasH = original.clone();
    
    std::vector<Vec2f> lines;
    HoughLines(binaria,lines,
            1,     //delta rho
            PI/180, //delta theta
            136);   //votos necessários
    drawLines(lines, linhasH);
    
    
    Mat linhasHP = original.clone();
    std::vector<Vec4i> linesP;
    HoughLinesP(binaria,linesP,
                    1, //delta rho
                    PI/180, //delta theta
                    136, //mínimo de votos necessários
                    100, //tamanho mínimo
                    20 //gap máximo
            );
    drawLinesP(linesP, linhasHP);
    
    imshow("original", original); waitKey(0);
    imshow("edges", edges); waitKey(0);
    imshow("bnarizada", binaria); waitKey(0);
    imshow("Hough", linhasH); waitKey(0);
    imshow("Hough P", linhasHP); waitKey(0);
    
       
       
    return 0;
}


void drawLines(std::vector<Vec2f> lines, Mat result){
    
    std::vector<cv::Vec2f>::const_iterator it= lines.begin();
    while (it!=lines.end()) {

            float rho= (*it)[0];   
            float theta= (*it)[1]; 

            if (theta < PI/4. || theta > 3.*PI/4.) { // ~vertical line

                    // point of intersection of the line with first row
                    cv::Point pt1(rho/cos(theta),0);        
                    // point of intersection of the line with last row
                    cv::Point pt2((rho-result.rows*sin(theta))/cos(theta),result.rows);
                    // draw a white line
                    cv::line( result, pt1, pt2, cv::Scalar(255), 1); 

            } else { // ~horizontal line

                    // point of intersection of the line with first column
                    cv::Point pt1(0,rho/sin(theta));        
                    // point of intersection of the line with last column
                    cv::Point pt2(result.cols,(rho-result.cols*cos(theta))/sin(theta));
                    // draw a white line
                    cv::line( result, pt1, pt2, cv::Scalar(255), 1); 
            }
            ++it;
    }
}

void drawLinesP(std::vector<Vec4i> lines, Mat image){
    std::vector<cv::Vec4i>::const_iterator it2= lines.begin();

    while (it2!=lines.end()) {

            cv::Point pt1((*it2)[0],(*it2)[1]);        
            cv::Point pt2((*it2)[2],(*it2)[3]);

            cv::line( image, pt1, pt2, cvScalar(255));

            ++it2;	
    }
}